@@ -8,23 +8,30 @@ Each function name BLAH_command implements a sub-command.  Then you use
 lamson.args.parse_and_run_command to parse the command line and run the function
 that matches.
 
-In your command function you get a dict of all the command line arguments the
-user gave.  Even if your command doesn't want them, you get them.  There's none
-of this crap of exactly specifying everything the user can and cannot do.  They
-give options, you figure out if they're right, and lamson.args just parses
-everything using a regular parser setup.
+Note that the _command suffix is optional and configurable, but it is there
+to disambiguate your commands so you can use Python reserved words and base
+types as your command names.  Without it, you can do a list_command or a
+for_command.
 
-In your command you'll want to enforce some arguments as defaults, and others as
-required.  You do that with lamson.args.defaults.  Pass in the options you were
-given, and keyword arguments for what are default, and what don't have defaults
-(are required).  For example, if you do:
+You command then specifies its keyword arguments to indicate what has
+reasonable defaults and what is required.  Give a value to the option
+to indicate its default, and give a None setting to indicate it is required.
+A good way to read this is it is your commands "default settings" and None
+says "this option has no default setting".
 
-    args.defaults(options, destroy="optparse", horribly=True, when=None)
+Here's an example from lamson:
 
-Then by default your command will destroy optparse horribly, but you have to say
-when with --when Sunday or something like that.  Also notice that if you give a
-boolean then your users can indicate that boolean is true by giving just the
-option (-horribly) or specify the truth (-horribly False).
+
+   def send_command(port=8825, host='127.0.0.1', debug=1, sender=None, to=None,
+                 subject=None, body=None, file=False):
+
+You can see this has subject, body, sender, and to as required options (they 
+are None), and the rest have some default value.
+
+With this the argument parser will parse the users given arguments, and then
+call your command function with those as keyword arguments, but after it has
+fixed them up with the defaults you gave.  In the event that a user does
+not give a required option, lamson.args will abort with an error telling them.
 
 Lamson's argument parser also accurately detects and parses integers, boolean
 values, strings, emails, single word values, and can handle trailing arguments
@@ -33,7 +40,7 @@ the right type for what you expect.
 
 Lamson.args does not care if you use one dash (-help), two dashes
 (--help), three dashes (---help) or a billion.  In all honesty, who gives a
-rat's ass, just get them to type something like a dash followed by a word and
+rat's ass, just get the user to type something like a dash followed by a word and
 that's good enough.
 
 If you just need argument parsing and no commands then you can just use
@@ -45,6 +52,7 @@ string of the function.
 
 import re
 import sys
+import inspect
 
 
 def s_ip_address(scanner, token): return ['ip_address', token]
@@ -150,9 +158,7 @@ def tokenize(argv):
     for arg in argv:
         toks, remainder = scanner.scan(arg)
         if remainder or len(toks) > 1:
-            # if we got a remainder then this is too complex so it's a generic
-            # string and they deal with it
-            tokens.append(['string', arg])
+            raise RuntimeError("You have triggered an unexpected condition.  Please tell the author.")
         else:
             tokens += toks
     return tokens
@@ -182,41 +188,46 @@ def parse(argv):
 
     print
 
-def defaults(options, **reqs):
+def determine_kwargs(function):
     """
-    Given a set of kw requirements, it will make sure that the options map given
-    has any required settings, and any that have default values get these set.
-    
-    If you want an argument be required, then set it to None in reqs:
-
-        args.defaults(options, cheese=None)
-
-    Then the user will be required to set chees to something.
-
-    If you want an argument to get a default, then give it a value that's not
-    None.
-
-        args.defaults(options, meat=1)
-
-    Then if they don't set meat, it gets set to 1.
-
-    Anything you don't specify is just ignored.
+    Uses the inspect module to figure out what the keyword arguments
+    are and what they're defaults should be, then creates a dict with
+    that setup.  The results of determine_kwargs() is typically handed
+    to ensure_defaults().
     """
+    spec = inspect.getargspec(function)
+    keys = spec[0]
+    values = spec[-1]
+    result = {}
+    for i in range(0,len(keys)):
+        result[keys[i]] = values[i]
+    return result
 
+def ensure_defaults(options, reqs):
+    """
+    Goes through the given options and the required ones and does the
+    work of making sure they match.  It will raise an ArgumentError
+    if any option is required.  It will also detect that required TRAILING
+    arguments were not given and raise a separate error for that.
+    """
     for key in reqs:
         if reqs[key] == None:
             # explicitly set to required
             if not options.has_key(key):
-                print "DEFAULTS: ", " ".join(["-%s %r" % (key, value) for key, value in reqs.items()])
-                raise ArgumentError("Option -%s is required by this command." % key)
+                if key == "TRAILING":
+                    raise ArgumentError("Additional arguments required after a -- on the command line.")
+                else:
+                    raise ArgumentError("Option -%s is required by this command." % key)
         else:
-            # given a default, so set that up
             if not options.has_key(key):
                 options[key] = reqs[key]
 
 def command_module(mod, command, options, ending="_command"):
     """Takes a module, uses the command to run that function."""
-    return mod.__dict__[command+ending](options)
+    function = mod.__dict__[command+ending]
+    kwargs = determine_kwargs(function)
+    ensure_defaults(options, kwargs)
+    return function(**options)
 
 
 def available_help(mod, ending="_command"):
@@ -290,7 +301,8 @@ def parse_and_run_command(argv, mod, default_command=None, exit_on_error=True):
         command_module(mod, command, options)
     except ArgumentError, exc:
         print "ERROR: ", exc
-        sys.exit(1)
+        if exit_on_error:
+            sys.exit(1)
 
     return True
 
