@@ -3,6 +3,26 @@ Implements the Lamson command line tool's commands, which are run
 by the lamson.args module dynamically.  Each command has it's
 actual user displayed command line documentation as the __doc__
 string.
+
+You will notice that all of the command functions in this module
+end in _command.  This is not required by the lamson.args module
+but it is the default.  You could easily use any other suffix, or
+none at all.
+
+This is done to disambiguate the command that it implements
+so that your command line tools do not clash with Python's
+reserved words and built-ins.  With this design you can have a
+list_command without clashing with list().
+
+You will also notice that commands which take trailing positional
+arguments give a TRAILING=[] or TRAILING=None (if it's required).
+This is done instead of *args because we need to use None to indicate
+that this command requires positional arguments.  TRAILING=[] is 
+like saying they are optional (but expected), and TRAILING=None is
+like saying they are required.  You can't (afaik) do TRAILING=None
+with *args.
+
+See lamson.args for more details.
 """
 
 from lamson import server, args, utils, queue, mail, routing
@@ -19,7 +39,8 @@ from pkg_resources import resource_stream
 from zipfile import ZipFile
 
 
-def log_command(options):
+def log_command(port=8825, host='127.0.0.1', debug=1, queue="", chroot=False,
+                cd=".", uid=False, gid=False, umask=False, pid="./run/log.pid"):
     """
     Runs a logging only server on the given hosts and port.  It logs
     each message it receives and also stores it to the run/queue 
@@ -29,25 +50,22 @@ def log_command(options):
             -pid ./run/log.pid -chroot False  \\
             -cd "." -umask False -uid False -gid False
     """
-    args.defaults(options, port=8825, host='127.0.0.1', debug=1, queue="",
-             chroot=False, cd=".", uid=False, gid=False, umask=False, pid="./run/log.pid")
-
-    if os.path.exists(options['pid']):
-        print "PID file %s exists, delete it first." % options['pid']
+    if os.path.exists(pid):
+        print "PID file %s exists, delete it first." % pid
         sys.exit(1)
 
-    utils.daemonize(options, [])
+    utils.daemonize(pid, cd, chroot, uid, gid, umask, files_preserve=[])
     sys.path.append(os.getcwd())
     logging.basicConfig(filename="logs/logger.log", level=logging.DEBUG)
 
     routing.Router.load(['lamson.handlers.log', 'lamson.handlers.queue'])
 
-    settings = utils.make_fake_settings(options)
-    settings.options = options
+    settings = utils.make_fake_settings(host, port)
     settings.receiver.start()
 
 
-def send_command(options):
+def send_command(port=8825, host='127.0.0.1', debug=1, sender=None, to=None,
+                 subject=None, body=None, file=False):
     """
     Sends an email to someone as a test message.
     See the sendmail command for a sendmail replacement.
@@ -55,22 +73,19 @@ def send_command(options):
     lamson send -port 8825 -host 127.0.0.1 -debug 1 \\
             -sender EMAIL -to EMAIL -subject STR -body STR -file False'
     """
-    args.defaults(options, port=8825, host='127.0.0.1', debug=1, sender=None,
-                  to=None, subject=None, body=None, file=False)
+    message = mail.MailResponse(From=sender,
+                                  To=to,
+                                  Subject=subject,
+                                  Body=body)
+    if file:
+        message.attach(file)
 
-    message = mail.MailResponse(From=options['sender'],
-                                  To=options['to'],
-                                  Subject=options['subject'],
-                                  Body=options['body'])
-    if options["file"]:
-        message.attach(options["file"])
-
-    relay = server.Relay(options['host'], port=options['port'],
-                         debug=options['debug'])
+    relay = server.Relay(host, port=port,
+                         debug=debug)
     relay.deliver(message)
 
 
-def sendmail_command(options):
+def sendmail_command(port=8825, host='127.0.0.1', debug=0, TRAILING=[]):
     """
     Used as a testing sendmail replacement for use in programs
     like mutt as an MTA.  It reads the email to send on the stdin
@@ -78,53 +93,48 @@ def sendmail_command(options):
 
     lamson sendmail -port 8825 -host 127.0.0.1 -debug 0 -- [recipients]
     """
-    args.defaults(options, port=8825, host='127.0.0.1', debug=0)
-
-    relay = server.Relay(options['host'], port=options['port'],
-                           debug=options['debug'])
+    relay = server.Relay(host, port=port,
+                           debug=debug)
     data = sys.stdin.read()
-    msg = mail.MailRequest(None, options['TRAILING'], None, data)
+    msg = mail.MailRequest(None, TRAILING, None, data)
     relay.deliver(msg)
 
 
 
-def syncdb_command(options):
+def syncdb_command(create=True):
     """
     Creates the database and other stuff for Lamson.
 
     lamson syncdb -create True
     """
-    args.defaults(options, create=True)
-
     settings = utils.import_settings(False, from_dir=os.getcwd())
     database = utils.configure_database(settings.database_config)
     database.create()
 
 
-def start_command(options):
+def start_command(pid='./run/smtp.pid', FORCE=False, chroot=False, cd=".",
+                  boot="config.boot", uid=False, gid=False, umask=False):
     """
     Runs a lamson server out of the current directory:
 
-    lamson start -pid ./run/smtp.pid -FORCE False -chroot False -cd "." -umask False -uid False -gid False
+    lamson start -pid ./run/smtp.pid -FORCE False -chroot False -cd "." \\
+            -umask False -uid False -gid False -boot config.boot
     """
-    args.defaults(options, pid='./run/smtp.pid', FORCE=False, chroot=False, cd=".", uid=False, gid=False, umask=False)
-
-    if os.path.exists(options["pid"]):
-        if not options["FORCE"]:
-            print "PID file %s exists, so assuming Lamson is running.  Give -FORCE to force it to start." % options["pid"]
+    if os.path.exists(pid):
+        if not FORCE:
+            print "PID file %s exists, so assuming Lamson is running.  Give -FORCE to force it to start." % pid
             sys.exit(1)
         else:
-            os.unlink(options["pid"])
+            os.unlink(pid)
 
-    utils.daemonize(options, [])
+    utils.daemonize(pid, cd, chroot, uid, gid, umask, files_preserve=[])
 
     sys.path.append(os.getcwd())
     settings = utils.import_settings(True, from_dir=os.getcwd())
-    settings.options = options
     settings.receiver.start()
 
 
-def stop_command(options):
+def stop_command(pid='./run/smtp.pid', KILL=False, ALL=False):
     """
     Stops a running lamson server.  Give -KILL True to have it
     stopped violently.  The PID file is removed after the 
@@ -133,17 +143,15 @@ def stop_command(options):
 
     lamson stop -pid ./run/smtp.pid -KILL False -ALL False
     """
-    args.defaults(options, pid='./run/smtp.pid', KILL=False, ALL=False)
-
     pid_files = []
 
-    if options["ALL"]:
-        pid_files = glob.glob(options["ALL"] + "/*.pid")
+    if ALL:
+        pid_files = glob.glob(ALL + "/*.pid")
     else:
-        pid_files = [options["pid"]]
+        pid_files = [pid]
 
-        if not os.path.exists(options["pid"]):
-            print "PID file %s doesn't exist, maybe Lamson isn't running?" % options["pid"]
+        if not os.path.exists(pid):
+            print "PID file %s doesn't exist, maybe Lamson isn't running?" % pid
             sys.exit(1)
 
     print "Stopping processes with the following PID files: %s" % pid_files
@@ -155,7 +163,7 @@ def stop_command(options):
         print "Attempting to stop lamson at pid %d" % int(pid)
 
         try:
-            if options['KILL']:
+            if KILL:
                 os.kill(int(pid), signal.SIGKILL)
             else:
                 os.kill(int(pid), signal.SIGHUP)
@@ -165,35 +173,32 @@ def stop_command(options):
             print "ERROR stopping Lamson on PID %d: %s" % (int(pid), exc)
 
 
-def restart_command(options):
+def restart_command(**options):
     """
     Simply attempts a stop and then a start command.  All options for both
     apply to restart.  See stop and start for options available.
     """
 
-    stop_command(options)
+    stop_command(**options)
     time.sleep(2)
-    start_command(options)
+    start_command(**options)
 
 
-def status_command(options):
+def status_command(pid='./run/smtp.pid'):
     """
     Prints out status information about lamson useful for finding out if it's
     running and where.
 
     lamson status -pid ./run/smtp.pid
     """
-    args.defaults(options, pid='./run/smtp.pid')
-
-
-    if os.path.exists(options["pid"]):
-        pid = open(options["pid"]).readline()
+    if os.path.exists(pid):
+        pid = open(pid).readline()
         print "Lamson running with PID %d" % int(pid)
     else:
         print "Lamson not running."
 
 
-def help_command(options):
+def help_command(**options):
     """
     Prints out help for the commands. 
 
@@ -204,7 +209,7 @@ def help_command(options):
     lamson help -for STR
     """
     if "for" in options:
-        help = args.help_for_command(lamson.commands, options["for"])
+        help = args.help_for_command(lamson.commands, options['for'])
         if help:
             print help
         else:
@@ -214,39 +219,36 @@ def help_command(options):
         print "\n".join(args.available_help(lamson.commands))
 
 
-def queue_command(options):
+def queue_command(pop=False, get=False, keys=False, remove=False, count=False,
+                  clear=False, name="run/queue"):
     """
     Let's you do most of the operations available to a queue.
 
     lamson queue (-pop | -get | -remove | -count | -clear | -keys) -name run/queue
     """
-    args.defaults(options, pop=False, get=False, keys=False,
-                  remove=False, count=False, clear=False,
-                  name="run/queue")
+    q = queue.Queue(name)
 
-    q = queue.Queue(options["name"])
-
-    if options["pop"]:
+    if pop:
         key, msg = q.pop()
         if key:
             print "KEY: ", key
             print msg
-    elif options["get"]:
-        print q.get(options["get"])
-    elif options["remove"]:
-        q.remove(options["remove"])
-    elif options["count"]:
-        print "Queue %s contains %d messages" % (options["name"], q.count())
-    elif options["clear"]:
+    elif get:
+        print q.get(get)
+    elif remove:
+        q.remove(remove)
+    elif count:
+        print "Queue %s contains %d messages" % (name, q.count())
+    elif clear:
         q.clear()
-    elif options["keys"]:
+    elif keys:
         print "\n".join(q.keys())
     else:
         print "Give something to do.  Try lamson help -for queue to find out what."
         sys.exit(1)
         
 
-def routes_command(options):
+def routes_command(TRAILING=['config.testing'], path=os.getcwd(), testing=True):
     """
     Prints out valuable information about an application's routing configuration
     after everything is loaded and ready to go.  Helps debug problems with
@@ -261,9 +263,8 @@ def routes_command(options):
     lamson routes -- config.boot
 
     """
-    args.defaults(options, TRAILING=['config.testing'], path=os.getcwd(), testing=True)
-    modules = options['TRAILING']
-    sys.path += options['path'].split(':')
+    modules = TRAILING
+    sys.path += path.split(':')
 
     for module in modules:
         __import__(module, globals(), locals())
@@ -277,14 +278,13 @@ def routes_command(options):
         print "\n---"
 
 
-def gen_command(options):
+def gen_command(project=None):
     """
     Generates various useful things for you to get you started.
 
     lamson gen -project STR
     """
-    args.defaults(options, project=None)
-    project = options['project']
+    project = project
 
     if os.path.exists(project):
         print "Project %s exists, delete it first." % project
@@ -310,7 +310,7 @@ def gen_command(options):
             out.write(prototype.read(file))
             out.close()
 
-def spell_command(options):
+def spell_command(TRAILING=None, language="en_US"):
     """
     Runs the PyEnchant spell checker on the given file.  Use this to train
     the spell checker for unit test runs when new words show up.
@@ -320,9 +320,6 @@ def spell_command(options):
     It will run interactively and if you perform an edit on the file it will save
     your changes back to the file and make a backup named with .bak.
     """
-
-    args.defaults(options, TRAILING=None, language="en_US")
-
     sys.path.append(os.getcwd())
 
     try:
@@ -346,15 +343,15 @@ def spell_command(options):
         return False
 
     try:
-        enchant.Dict(options['language'])
+        enchant.Dict(language)
     except DictNotFoundError, exc:
         print "PyEnchant couldn't find your dictionary.  Put them in ~/.enchant/myspell."
         print exc
         return False
 
-    for file in options['TRAILING']:
+    for file in TRAILING:
         text = open(file).read()
-        chkr = SpellChecker(options['language'],filters=[EmailFilter,URLFilter]) 
+        chkr = SpellChecker(language,filters=[EmailFilter,URLFilter]) 
         chkr.set_text(text) 
         cmdln = CmdLineChecker() 
         cmdln.set_checker(chkr) 
@@ -368,7 +365,7 @@ def spell_command(options):
     return True
 
 
-def web_command(options):
+def web_command(basedir=".", port=8888, host='127.0.0.1'):
     """
     Starts a very simple files only web server for easy testing of applications
     that need to make some HTML files as the result of their operation.
@@ -382,11 +379,9 @@ def web_command(options):
     from SimpleHTTPServer import SimpleHTTPRequestHandler 
     import os
 
-    args.defaults(options, basedir=".", port=8888, host='127.0.0.1')
-
-    os.chdir(options['basedir'])
-    server = HTTPServer((options['host'], options['port']), SimpleHTTPRequestHandler)
+    os.chdir(basedir)
+    server = HTTPServer((host, port), SimpleHTTPRequestHandler)
     print "Starting server on %s:%d out of directory %r" % (
-        options['host'], options['port'], options['basedir'])
+        host, port, basedir)
     server.serve_forever()
 
