@@ -5,22 +5,21 @@ from lamson.testing import spelling
 from lamson import commands
 
 def test_send_command():
-    commands.send_command({'sender': 'test@test.com',
-                           'to': 'test@test.com',
-                           'body': 'Test body',
-                           'subject': 'Test subject',
-                           'file': 'setup.py',
-                           'port': 8899,
-                          })
+    commands.send_command(sender='test@test.com',
+                           to='test@test.com',
+                           body='Test body',
+                           subject='Test subject',
+                           file='setup.py',
+                           port=8899,)
 
 def test_status_command():
-    commands.status_command({})
-    commands.status_command({'pid': 'run/donotexist.pid'})
+    commands.status_command(pid='run/log.pid')
+    commands.status_command(pid='run/donotexist.pid')
 
 
 def test_help_command():
-    commands.help_command({})
-    commands.help_command({'for': 'status'})
+    commands.help_command()
+    commands.help_command(**{'for': 'status'})
 
 def test_queue_command():
     for op in  ['pop', 'count', 'clear', 'keys']:
@@ -31,14 +30,14 @@ def test_gen_command():
     if os.path.exists(project):
         shutil.rmtree(project)
 
-    commands.gen_command({'project':project})
+    commands.gen_command(project=project)
     assert os.path.exists(project)
     shutil.rmtree(project)
 
 
 def test_route_command():
-    commands.routes_command({'TRAILING': ['lamson.handlers.log',
-                                          'lamson.handlers.queue']})
+    commands.routes_command(TRAILING=['lamson.handlers.log',
+                                      'lamson.handlers.queue'])
 
 def test_spell_command():
     if 'PYENCHANT_LIBRARY_PATH' not in os.environ:
@@ -50,6 +49,7 @@ def test_spell_command():
     contents = open(template).read()
     assert spelling(template, contents), "%s has to be spelled right." % template
 
-    assert commands.spell_command({'TRAILING': ["tests/lamson_tests/template.txt"], 'language': 'en_US'})
+    assert commands.spell_command(TRAILING=["tests/lamson_tests/template.txt"],
+                                  language='en_US')
 
 
