@@ -21,6 +21,7 @@ def test_Filter():
     sf = spam.Filter(ham_db, 'tests/.hammierc')
     ham_msg = test_mail_request().msg
     spam_msg = test_mail_response_plain_text().as_message()
+    sf.newdb()
 
     sf.train_ham(ham_msg)
     sf.train_spam(spam_msg)
@@ -30,6 +31,10 @@ def test_Filter():
 
     sf.close()
 
+    sf = spam.Filter(ham_db, 'tests/.hammierc')
+    sf.open('r')
+
+
 def test_spam_filter():
     import spam_filtered_mod
 
@@ -38,13 +43,14 @@ def test_spam_filter():
     sf.train_spam(msg.msg)
 
     Router.deliver(msg)
-    assert Router.in_state(spam_filtered_mod.START, msg), "Spam got through"
+    assert Router.in_state(spam_filtered_mod.SPAMMING, msg), "Spam got through"
 
-    print Router.STATE_STORE.states
-    print Router.REGISTERED
+    Router.clear_states()
     sf.untrain_spam(msg.msg)
     sf.train_ham(msg.msg)
 
     Router.deliver(msg)
+    print Router.STATE_STORE.states
+    print Router.REGISTERED
     assert Router.in_state(spam_filtered_mod.END, msg), "Ham didn't go through."
 
